/***************************************************************************
                          appwindow.cpp  -  description
                             -------------------
    begin                : Sun Jul 16 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: appwindow.cpp,v 1.8 2000/08/01 18:12:49 js Exp $
 *
 */


#include "k8ktest.h"

ApplicationWindow::ApplicationWindow(QString conn)
    : QMainWindow( 0, "MainWindow" )
{
	int i=0;
	int posw=45;
	int buttonw=80;
	int buttonh=25;

	/* create the settins class */
	settings = new Settings();
	
	/* set the permissions on port */
	if(settings->lptport==1)
	{
		if(ioperm(0x378, 3, 1))
		{
			QMessageBox::message("Error","Could not set permissions on printerport 1(remember to run as root)");
			exit(1);
		}
	}
	else
	{
		if(ioperm(0x278, 3, 1))
		{
			QMessageBox::message("Error","Could not set permissions on printerport 2(remember to run as root)");
			exit(1);
		}
	}

	SelectI2CprinterPort(settings->lptport);
	I2CbusDelay=settings->i2cbusdelay;

	/* init the vars */
	debugstate = FALSE;
	debugwindowstate = FALSE;
			
	/* make a menu */
	this->menu = new MenuBar(this,"menu");

	/* create the toolbar */
	toolbar = new ToolBar(this,"toolbar");
	this->addToolBar(toolbar);

	/* create the statusbar */
	statusbar = new StatusBar(this,"statusbar");
		
	/* Create the debugdialog */
	this->debugdialog = new DebugDialog(this,"Debugdialog");

	/* Create the optionsdialog */
	this->optionsdialog = new OptionsDialog(this,"optionsdialog");
	
	/* Create the mainframe and set as central widget */
	mainframe = new QFrame(this,"mainframe");
	this->setCentralWidget(mainframe);
	
	/* Create the analogoframe to surround the analog outputs */
	analogoframe = new QFrame(mainframe,"analogoframe");
	
	/* Create the analogoframe to surround the analog outputs */
	analogiframe = new QFrame(mainframe,"analogiframe");
	
	/* Create the analogoframe to surround the analog outputs */
	digitalioframe = new QFrame(mainframe,"digitalioframe");

	/* Create the buttonframe to surround the analog outputs */
	buttonframe = new QFrame(mainframe,"buttonframe");
	buttonframe->setGeometry(640,165,120,200);
	
	/*Create the buttons*/
	alli = new QPushButton(buttonframe,"alli");
	alli->setText("All input");
	alli->setGeometry(0,0,buttonw,buttonh);
	
	allo = new QPushButton(buttonframe,"allo");
	allo->setText("All output");
	allo->setGeometry(0,30,buttonw,buttonh);
	
	set_max = new QPushButton(buttonframe,"set_max");
	set_max->setText("All up");
	set_max->setGeometry(0,60,buttonw,buttonh);
	
	set_min = new QPushButton(buttonframe,"set_min");
	set_min->setText("All down");
	set_min->setGeometry(0,90,buttonw,buttonh);
	
	quit = new QPushButton(buttonframe,"quit");
	quit->setText("Quit");
	quit->setGeometry(0,120,buttonw,buttonh);
	
	/* Create a digitalios  */
	for (i=0;i<16;i++)
	{
		digitalio[i] = new DigitalIo(0,i+1,QString("IO")+tr("%1").arg(i+1),digitalioframe,"io");
		digitalio[i]->SetGeometry(i*posw,0,posw,100,18);
		digitalio[i]->SetIoType(FALSE);
	}
	digitalioframe->setGeometry(5,5,i*posw,100);
	
	/* Create a analogoutputs */
	for (i=0;i<9;i++)
	{
		if (i<8)
			analogoutput[i] = new AnalogOutput(0,63,1,10,0,i+1,QString("DAC")+tr("%1").arg(i+1),analogoframe,"io");
		/* the last output has 8 bit resolution */
		else
			analogoutput[i] = new AnalogOutput(0,255,1,10,0,i+1,QString("DA")+tr("%1").arg(i+1),analogoframe,"io");
			
		analogoutput[i]->SetGeometry(i*posw,0,posw,200,18);
	}
	analogoframe->setGeometry(5,digitalioframe->height()+10,i*posw,200);
	
	/* Create a analoginputs  */
	for (i=0;i<4;i++)
	{
		analoginput[i] = new AnalogInput(0,255,1,10,0,i+1,QString("AD")+tr("%1").arg(i+1),analogiframe,"io");
		analoginput[i]->SetGeometry(i*posw,0,posw,200,18);
	}
	analogiframe->setGeometry(5+analogoframe->width()+posw,digitalioframe->height()+10,i*posw,200);

	/* create a timer to scan the k8000 card */
	timer = new QTimer( this );
	
	/* Connect the slots and signals */
	QObject::connect( debugdialog, SIGNAL(Closed()), this, SLOT(HideDebugDialog()) );
	QObject::connect( timer, SIGNAL(timeout()),this, SLOT(Scan()) );
	QObject::connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
	QObject::connect( allo, SIGNAL(clicked()), this, SLOT(AllOutput()) );
	QObject::connect( alli, SIGNAL(clicked()), this, SLOT(AllIntput()) );
	QObject::connect( set_min, SIGNAL(clicked()), this, SLOT(SetMinimum()) );
	QObject::connect( set_max, SIGNAL(clicked()), this, SLOT(SetMaximum()) );

	/* restore the settings */
	this->RestoreSettings();
	
	/* size the main window */
	this->resize(730,400);

	/* start the scan timer */
	timer->start( settings->refreshrate, FALSE );
	
	/* we're ready */
	statusbar->SetStatusString(QString("Ready"));
}

ApplicationWindow::~ApplicationWindow()
{}

void ApplicationWindow::Scan()
{
	int i;
	
	this->statusbar->Blink();	
	/* refresh all io */
	for (i=0;i<16;i++)
		digitalio[i]->Refresh();
	for (i=0;i<9;i++)
		analogoutput[i]->Refresh();
	for (i=0;i<4;i++)
		analoginput[i]->Refresh();
}

void ApplicationWindow::RestoreSettings()
{
	int i;
	/* call restore to the settings */
	settings->RestoreSettings();
	for (i=0;i<9;i++)
		analogoutput[i]->SetValue(settings->aostate[i]);
	for (i=0;i<16;i++)
	{
		digitalio[i]->SetIoType(settings->iotype[i]);
		digitalio[i]->SetValue(settings->iostate[i]);	
	}
	this->SetNewOptions();	
}

void ApplicationWindow::SaveSettings()
{
	/* call save to the settings object*/
	int i;
	for (i=0;i<9;i++)
		settings->aostate[i] = analogoutput[i]->GetValue();
	for (i=0;i<16;i++)
	{
		settings->iotype[i]  = digitalio[i]->GetIoType();
		settings->iostate[i] = digitalio[i]->GetValue();	
	}
	settings->SaveSettings();
}


void ApplicationWindow::AllOutput()
{
	int i;
	for (i=0;i<16;i++)
		digitalio[i]->SetIoType(FALSE);	
}

void ApplicationWindow::AllIntput()
{
	int i;
	for (i=0;i<16;i++)
		digitalio[i]->SetIoType(TRUE);	
}

void ApplicationWindow::SetMinimum()
{
	int i;
	for (i=0;i<9;i++)
		analogoutput[i]->SetValue(0);
	for (i=0;i<16;i++)
		digitalio[i]->SetValue(FALSE);	

}

void ApplicationWindow::SetMaximum()
{
	int i;
	for (i=0;i<9;i++)
	{
		if(i<8)
			analogoutput[i]->SetValue(63);
		else
			analogoutput[i]->SetValue(254);
	}
	for (i=0;i<16;i++)
		digitalio[i]->SetValue(TRUE);	
}

	
void ApplicationWindow::ShowAbout()
{		
	AboutDialog about(this,"About");
	about.exec();
}

void ApplicationWindow::InsertDebugText(QString debugstring)
{
	if(this->debugstate)
		this->debugdialog->InsertDebugText(QString(debugstring));
}

void ApplicationWindow::SetUnsetDebug()
{
	this->debugstate = !this->debugstate;
	this->menu->SetDebugState(this->debugstate);
}

void ApplicationWindow::ShowDebugDialog()
{
	this->debugwindowstate = TRUE;
	this->debugdialog->show();
	this->menu->SetDebugWindowState(this->debugwindowstate);
}

void ApplicationWindow::ShowHideDebugDialog()
{
	this->debugwindowstate = !this->debugwindowstate;
	if(this->debugwindowstate)
		this->debugdialog->show();
	else
		this->debugdialog->hide();
	this->menu->SetDebugWindowState(this->debugwindowstate);
}

void ApplicationWindow::HideDebugDialog()
{
	this->debugwindowstate = FALSE;
	this->debugdialog->hide();
	this->menu->SetDebugWindowState(this->debugwindowstate);
}

void ApplicationWindow::ShowOptionDialog()
{
	OptionsDialog od(this,"Login");
	if ( od.exec() )
	{
		settings->refreshrate = od.GetRefreshrate();
		settings->i2cbusdelay = od.GetI2cBusDelay();
		settings->lptport     = od.GetLptport();
	}
	this->SetNewOptions();

}

void ApplicationWindow::SetNewOptions()
{
	timer->changeInterval( settings->refreshrate );
	
	if(settings->lptport==1)
	{
		if(ioperm(0x378, 3, 1))
		{
			QMessageBox::message("Error","Could not set permissions on printerport (remember to run as root)");
			exit(1);
		}
	}
	else
	{
		if(ioperm(0x278, 3, 1))
		{
			QMessageBox::message("Error","Could not set permissions on printerport (remember to run as root)");
			exit(1);
		}
	}
	SelectI2CprinterPort(settings->lptport);
	I2CbusDelay=settings->i2cbusdelay;
}

void ApplicationWindow::NullSlot()
{/* This method does nothing (as it's name sugests) */}

